% MLE of the signal to noise ratio
clear all; clc;
cpi = xlsread('uscpi.xls');
% US monthly inflation rate
vy = 100 * diff(log( cpi( (1960-1947)*12:end) ) );
plot(vy);
vy = diff(vy); % second differences
plot(vy);
% ------------------------------------------------------------------------
% initial value transformed hyperparameter: theta 
cp = 1; cq =1;
vP0 = zeros(cp+cq,1);  
f  = @(vP)SsfLogLikConc(vP, vy, cp, cq); % function 
opts = optimset('Display','iter','TolX',1e-9,'TolFun',1e-9,...
                'Diagnostics','off', 'MaxIter',10000, 'MaxFunEvals', 10000,...
                'LargeScale', 'off', 'PlotFcns', @optimplotfval);
[vP, fval, exitflag, output] = fminunc(f, vP0,opts);
vP
 
disp('Estimation results');
if (cp > 0) 
    vphi(1:cp) = ReparAR(FisherInvTransform(vP(1:cp)));
    disp(['Autoregressive coefficients: ', num2str(vphi )]);
end
if (cq > 0) 
    vtheta(1:cq) = ReparMA(FisherInvTransform(vP(cp+1:cp+cq)));
    disp(['Moving average coefficients: ', num2str(vtheta )]); 
end
  
[mZ, mGG, mT, mHH, mHG, va, mP] = SetStateSpaceModel(vP, cp, cq) 
[vInnovations, vF, mStatePred, mCovStatePred,...
    dLogLik, dSigma2, dLogLikConc, dPev] = KalmanFilter(vy, mZ, mGG, mT, mHH, mHG, va, mP);
vStandInn = vInnovations ./ sqrt(dSigma2 * vF);
subplot(2,1,1),
cn = length(vy);
plot(1:cn, vStandInn); title('Standardised Innovations');
subplot(2,1,2),
autocorr(vStandInn(2:end), 20); 
disp(['Prediction error variance: ', num2str(dSigma2)]);
disp(['AIC: ', num2str(-2*dLogLik + 2 * (cp+cq))]);