% MLE of the signal to noise ratio
% Estimation of Level component
clear all; clc, clf;
cpi = xlsread('uscpi.xls');
% US monthly inflation rate
vy = 100 * diff(log( cpi( (1960-1947)*12:end) ) );
plot(vy);
cn = length(vy);
% ------------------------------------------------------------------------
% initial value transformed hyperparameter: theta = ln sqrt q
theta0 = 0;  
f  = @(theta)SsfLogLikConc(theta, vy); % function 
[theta, fval, exitflag, output] = fminunc(f, theta0);
q = exp(2 * theta); 
disp(['MLE of signal noise ratio is ', num2str(q)]);

%-------------------------------------------------------------------------
[mZ, mGG, mT, mHH, va, mP] = SetStateSpaceModel(theta);

[vInn, mF, mStatePred, mCovStatePred,dLogLik, dSigma2, dLogLikConc, dPev] = KalmanFilter(vy, mZ, mGG, mT, mHH, va, mP);
disp(['Irregular Disturbance variance ', num2str(dSigma2)]);
disp(['Level Disturbance variance ', num2str(dSigma2 * q)]);
vStandInn = vInn ./ sqrt(dSigma2 * mF);

subplot(2,1,1),
plot(1:cn, vStandInn); title('Standardised Innovations');
subplot(2,1,2),
autocorr(vStandInn(2:end), 20); 
clf;

[vSmoothedLevel, vVarSLevel] = KFStateSmoother(vy, mZ, mGG, mT, mHH, va, mP);
cn = length(vy);
plot(1:cn, vy, 'r', 1:cn, vSmoothedLevel, 'b');
[vRealTimeLevel, vVarRTLevel] = KFRealTime(vy, mZ, mGG, mT, mHH, va, mP);
plot(1:cn, vy, 'r', 1:cn, vRealTimeLevel, 'b');

subplot(2,1,1), 
plot(1:cn, vy, 'r', ...
     1:cn, vSmoothedLevel, 'b', ...
     1:cn, vSmoothedLevel+1.96 * sqrt(dSigma2 * vVarSLevel), '-g',...
     1:cn, vSmoothedLevel-1.96 * sqrt(dSigma2 * vVarSLevel), '-g'),
title('Smoothed Estimates');
subplot(2,1,2), 
plot(1:cn, vy, 'r', ...
     1:cn, vRealTimeLevel, 'b', ...
     1:cn, vRealTimeLevel+1.96 * sqrt(dSigma2 * vVarRTLevel), '-g',...
     1:cn, vRealTimeLevel-1.96 * sqrt(dSigma2 * vVarRTLevel), '-g');
 
 
