function [mZ, mGG, mT, mHH, va, mP] = SetStateSpaceModel(vP)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Trend plus cycle model 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Assign hyperparameters 
dSkappa = 1.0;  % concentrated out of LF
dSeta =  exp(2.0 * vP(1));      
dSzeta =  exp(2.0 * vP(2));         
drho = abs(vP(3))/sqrt(1+vP(3)^2);  % damping factor
dlambda = (2*pi)/(2+exp(vP(4)));	% cycle frequency
%--------------------------------------------------------------------------
cm = 4;     % number of state elements
% Measurement equation
% y_{t} = Z_t \alpha_{t} +  G_t \epsilon_t
mZ = [1 0 1 0];     
mGG = 0; % G_t * G_t'
% Transition equation
% \alpha_{t+1} = T_t \alpha_{t} + H_t \epsilon_t
mTtrend = [1 1; 0 1]; 
mTcycle = drho * [cos(dlambda) sin(dlambda); -sin(dlambda) cos(dlambda)]; 
mT = blkdiag(mTtrend, mTcycle);     
mHH = diag([dSeta; dSzeta; dSkappa; dSkappa]); % H_t * H_t'  
%%%%%%%%%%%%%%% Initial conditions               %%%%%%%%%%%%%%%%%%%%%%%%%
va = zeros(cm, 1);    
mP = blkdiag(10^9 * eye(2), dSkappa / (1-drho^2) * eye(2) );     
end


