% MLE of Trend plus Cycle Model for quarterly US GDP
clear all; clc; 
gdp = xlsread('GDPC96.xls');
% US quarterly GDP
vy = 400 * log(gdp);
cn = length(vy);
startdate = 1947;	% date range is from 1947:1 to 2011:2 (quarterly)
enddate = 2012;	% change these two lines to try alternative periods
 
gdpdates =  startdate:0.25:enddate;

plot(gdpdates, vy);
% ------------------------------------------------------------------------
% initial value transformed hyperparameter:   
vP0 = [-9; -0.5; 0.5; 2];  
f  = @(vP)SsfLogLikConc(vP, vy); % function 
opts = optimset('Display','iter','TolX',1e-9,'TolFun',1e-9,...
                'Diagnostics','off', 'MaxIter',10000, 'MaxFunEvals', 10000,...
                'LargeScale', 'off', 'PlotFcns', @optimplotfval);
[vP, fval, exitflag, output] = fminunc(f, vP0,opts);
 
disp(['MLE of transformed parameters:  ', num2str(vP')]);
disp(['Likelihood:  ', num2str(-1.0*fval)]);
%-------------------------------------------------------------------------
[mZ, mGG, mT, mHH, va, mP] = SetStateSpaceModel(vP);
[vInn, mF, mStatePred, mCovStatePred, dLogLik, dSigma2, ...
    dLogLikConc, dPev] = KalmanFilter(vy, mZ, mGG, mT, mHH, va, mP);
dSkappa = dSigma2; 
dSeta =  dSigma2 * exp(2.0 * vP(1));      
dSzeta =  dSigma2 * exp(2.0 * vP(2));         
drho = abs(vP(3))/sqrt(1+vP(3)^2);  % damping factor
dlambda = (2*pi)/(2+exp(vP(4)));	% cycle frequency
disp('ESTIMATION RESULTS         ');
disp(['Level Disturbance variance ', num2str(dSeta)]);
disp(['slope Disturbance variance ', num2str(dSzeta)]);
disp(['Cycle Disturbance variance ', num2str(dSkappa)]);
disp(['Cycle frequency            ', num2str(dlambda)]);
disp(['Cycle period               ', num2str(2*pi/dlambda), ' ( ', num2str(2*pi/dlambda/4), ' years)']);
disp(['Cycle damping factor       ', num2str(drho)]);
vStandInn = vInn ./ sqrt(dSigma2 * mF);

subplot(2,1,1),
plot(1:cn, vStandInn); title('Standardised Innovations');
subplot(2,1,2),
autocorr(vStandInn(3:end), 20); 

clf;
[mState, mVarState] = KFStateSmoother(vy, mZ, mGG, mT, mHH, va, mP);
vTrend = mState(1,:);
vVarTrend = mVarState(1,:);
vCycle = mState(3,:);
vVarCycle = mVarState(3,:);
plot(1:cn, vy, 'r', 1:cn, vTrend, 'b');
 

subplot(2,1,1), 
plot(1:cn, vy, 'r', ...
     1:cn, vTrend, 'b', ...
     1:cn, vTrend+1.96 * sqrt(dSigma2 * vVarTrend), '-g',...
     1:cn, vTrend-1.96 * sqrt(dSigma2 * vVarTrend), '-g'),
title('Smoothed Estimates: trend component');
subplot(2,1,2), 
plot(1:cn, vCycle, 'b', ...
     1:cn, vCycle+1.96 * sqrt(dSigma2 * vVarCycle), '-g',...
     1:cn, vCycle-1.96 * sqrt(dSigma2 * vVarCycle), '-g');
 
%------------------------------------------------------------------
% Hodrick and Prescott trend and cycle
clf;
[HPtrend, HPcycle] = hpfilter(vy, 1600);
subplot(2,1,1), plot(gdpdates(2:end), diff(vy), '.b', ...
                     gdpdates(2:end), diff(vTrend), 'r', ...
                     gdpdates(2:end), diff(HPtrend), 'g'), ...
    title('1st differences series and trend');
subplot(2,1,2), plot(gdpdates, vCycle, 'b',  gdpdates, HPcycle,'r' ), ...
    title('UC Cycle vs HP Cycle');
